#__perl__

=pod

=head1 Comp670.pm

A package of perl methods written for the Computational Sciences 670 class, 2006 by Rob Edwards, with some help from Peter Salalmon.

The easiest way to use this module is to include it in your perl script like this:

    use lib '.';
    use Comp670;

    my $comp=new Comp670;

The easiest way to read this documentation is to use the command 
    
    perldoc Comp670.pm

=head2 File Locations

The program will try and guess the locations of the files it needs.  It will look at these locations:

    .
    ..
    ../MinimalVersion
    MinimalVerion/

You should make sure that the files proteins.txt, length_results.txt, recombined_data.txt are available in one of those places.

=head2 About

Written by Rob Edwards (redwards@salmonella.org), this package contains some methods for accessing the phage proteome tree data as flat files.

Copyright Rob Edwards 2006, released under the same terms as PERL itself.

=head2 Version

Version 1. November 22nd, 2006

=head1 new()

Instantiate a new method. At the moment all this does is bless some variables, and doesn't actually predefine anything, however you can set up this method to do all sorts of nice calculations for you.

You should call this method when you want to start using this package like this:

    my $comp=new Comp670;  
    or
    my $comp=Comp670->new();

=cut

use strict;
package Comp670;

sub new {
    my ($class, %arg) = @_;
    
    my $self = bless{},$class;
    
    return $self;
}


=head1 protein2genome()

Convert a protein number to a genome number. We read the whole file into memory as a hash, so the first iteration is slow, but the subsequent ones are very quick.

Use this method like this:

    my $genome=$comp->protein2genome(123);


=cut

sub protein2genome {
    my ($self, $p)=@_;
    return unless ($p);
    unless ($self->{'protein2genome'})
    {
        # we need to read the file. We're going to start by guessing some of the locations where it may be.
        my $file;
        foreach my $t ("proteins.txt", "../proteins.txt", "../MinimalVersion/proteins.txt", "MinimalVersion/proteins.txt")
        {
            if (-e $t) {$file=$t; last}
        }
        
        unless ($file)  {die "Sorry, I couldn't find the file proteins.txt. Please check the location of it, and try again\n"}
        
        open(IN, $file) || die "Can't open $file";
        while (<IN>)
        {
            next if (/^\#/); # ignore comments
            chomp;
            s/\s+$//; # there is a weird file conversion thing where there is a lot of space or something weird at the end of each line
            my @a=split /\t/;
            $self->{'protein2genome'}->{$a[0]}=$a[1];
        }
        close IN;
    }
    return $self->{'protein2genome'}->{$p};
}

=head1 genome2protein()

Get all the proteins for a given genome. This will return a reference to an array, so you will need to dereference it before using it.

Use the method like this
        
    my $proteins=$comp->genome2protein(2);
    foreach my $protein (@$proteins) 
    {
        ...
    }

=cut

sub genome2protein {
    my ($self, $g)=@_;
    return unless ($g);
    unless ($self->{'genome2protein'})
    {
        # we need to read the file. We're going to start by guessing some of the locations where it may be.
        my $file;
        foreach my $t ("proteins.txt", "../proteins.txt", "../MinimalVersion/proteins.txt", "MinimalVersion/proteins.txt")
        {
            if (-e $t) {$file=$t; last}
        }
        
        unless ($file)  {die "Sorry, I couldn't find the file proteins.txt. Please check the location of it, and try again\n"}
        
        open(IN, $file) || die "Can't open $file";
        while (<IN>)
        {
            next if (/^\#/); # ignore comments
            chomp;
            s/\s+$//; # there is a weird file conversion thing where there is a lot of space or something weird at the end of each line
            my @a=split /\t/;
            push @{$self->{'genome2protein'}->{$a[1]}}, $a[0];
        }
        close IN;
    }
    return $self->{'genome2protein'}->{$g};
}
        
=head1 protein_distance()

Retrive the protein distance score for any two proteins. Given any two proteins retrieves the protein distance. Returns an undefined value if the score is "infinite".

Use like this:

    my $pdist=$comp->protein_distance($p1, $p2);

It doesn't matter which order you provide p1 and p2

=cut

sub protein_distance {
    my ($self, $p1, $p2)=@_;
    return unless ($p1 && $p2);

    unless ($self->{'protdist'})
    {
        # we need to read the file. We're going to start by guessing some of the locations where it may be.
        my $file;
        foreach my $t ("recombined_data.txt", "../recombined_data.txt", "../MinimalVersion/recombined_data.txt", "MinimalVersion/recombined_data.txt")
        {
            if (-e $t) {$file=$t; last}
        }

        open(IN, $file) || die "Can't open $file";
        while (<IN>)
        {
            next if (/^\#/); # ignore comments
            chomp;
            s/\s+$//; # there is a weird file conversion thing where there is a lot of space or something weird at the end of each line
            my @a=split /\t/;
            ($a[0] > $a[1]) ? (($a[0], $a[1])=($a[1], $a[0])) : 1; # if the first number is bigger than the second, switch them, else keep as they are
            $self->{'protdist'}->{$a[0]}->{$a[1]}=$a[3];
        }
    }

    ($p1 > $p2) ? (($p1, $p2)=($p2, $p1)) : 1; # if the first number is bigger than the second, switch them, else keep as they are
    return $self->{'protdist'}->{$p1}->{$p2} if ($self->{'protdist'}->{$p1}->{$p2});
    return undef;
}

=head1 length_information()

Get the information about the lengths for a pair of proteins. For a pair of proteins it will return an array with the following information:

    protein id1, Length 1, protein id2, length 2, length of alignment, # gaps

Use like this:

    my ($id1, $len1, $id2, $len2, $aln, $gaps)=$comp->length_information($p1, $p2);

If there is no information the array will contained undefined values for Length 1, length 2, length of alignment, and # gaps, but not for id1 and id2.

=cut

sub length_information {
    my ($self, $p1, $p2)=@_;
    return unless ($p1 && $p2);

    unless ($self->{'length'})
    {
        # we need to read the file. We're going to start by guessing some of the locations where it may be.
        my $file;
        foreach my $t ("length_results.txt", "../length_results.txt", "../MinimalVersion/length_results.txt", "MinimalVersion/length_results.txt")
        {
            if (-e $t) {$file=$t; last}
        }

        open(IN, $file) || die "Can't open $file";
        while (<IN>)
        {
            next if (/^\#/); # ignore comments
            next if (/^ID/); # ignore the first line of the file
            chomp;
            s/\s+$//; # there is a weird file conversion thing where there is a lot of space or something weird at the end of each line
            my @a=split /\t/;
            ($a[0] > $a[2]) ? (($a[0], $a[2])=($a[2], $a[0])) : 1; # if the first number is bigger than the second, switch them, else keep as they are
            $self->{'length'}->{$a[0]}->{$a[2]}=\@a;
        }
    }

    my ($r1, $r2)=($p1, $p2); # we need to remember which order they were asked in so we can return them in the same order.
    ($p1 > $p2) ? (($p1, $p2)=($p2, $p1)) : 1; # if the first number is bigger than the second, switch them, else keep as they are
    
    my @ret=($r1, undef, $r2, undef, undef, undef);
    if ($self->{'length'}->{$p1}->{$p2})
    {
        @ret=@{$self->{'length'}->{$p1}->{$p2}};
        if ($p1 != $r1) {@ret[0,1,2,3]=@ret[2,3,0,1]} # correct the order of the 
    }

    return @ret;
}




1;
        
